class VersionException(Exception):
    """ Custom Version Exception for logging exception
    """
    pass


class Version(object):
    """ Version class for maintaining the versions

        Example:
            version = Version(10.10.1-beta.2)
            print(version.major)
            >> 10
            print(version.preRelease)
            >> 2
            print(version)
            >> 10.10.1-beta.2
    """
    def __init__(self, version=None):
        """ Version initializer

            KwArgs:
                version (str): Version string to parse.(Default: "1.0.0")
        """
        self._version = None
        self.major = self.minor = self.patch = self.preRelName = self.preRelease = None
        self._acceptedPreRelNames = ['alpha', 'beta', 'rc']

        self.version = version or "1.0.0"

    def __str__(self):
        """ Override function to return the version string object
        """
        if self.is_pre_version and self.preRelName and self.preRelease:
            return "{major}.{minor}.{patch}-{preRelN}.{preRel}" .format(
                major=self.major,
                minor=self.minor,
                patch=self.patch,
                preRelN=self.preRelName,
                preRel=self.preRelease,
            )
        else:
            return "{major}.{minor}.{patch}" .format(
                major=self.major,
                minor=self.minor,
                patch=self.patch,
            )

    @property
    def version(self):
        return self._version

    @version.setter
    def version(self, ver):
        self._version = ver
        self._parse_version()

    @property
    def is_pre_version(self):
        """ Definition to check if the version is pre release or not

            :returns
                result (bool): Returns boolean value if the version is a pre release
        """
        result = False
        if '-' in self.version:
            result = True
        return result

    def _parse_version(self):
        """ Parse the version to specified types like major, minor, patch etc.,

            :raises VersionException if version syntax is wrong
        """
        if self.is_pre_version:
            pre_release_split = self.version.split('-')[-1].split('.')
            if not len(pre_release_split) == 2:
                raise VersionException(
                    "Pre release syntax must be <preReleaseName>.<preReleaseVersion>"
                )
            if not pre_release_split[0] in self._acceptedPreRelNames:
                raise VersionException(
                    "Pre release name must be of %s" % ", ".join(
                        self._acceptedPreRelNames
                    )
                )
            self.preRelName = pre_release_split[0]
            self.preRelease = self._get_int_value(pre_release_split[1], ver_type="pre")
        release_split = self.version.split('-')[0].split('.')
        if not len(release_split) == 3:
            raise VersionException(
                "Release syntax must be <major>.<minor>.<patch>"
            )
        self.major = self._get_int_value(release_split[0])
        self.minor = self._get_int_value(release_split[1])
        self.patch = self._get_int_value(release_split[2])

    @staticmethod
    def _get_int_value(version, ver_type=None):
        """ convert the string version to integer

            :raises VersionException if given version is a string
        """
        try:
            result = int(version)
        except ValueError:
            if ver_type:
                raise VersionException(
                    "Pre release version must be of type integer, not string"
                )
            else:
                raise VersionException(
                    "Release version must be of type integer, not string"
                )
        return result

    @property
    def version_info(self):
        version_dict = SemVersion(
            self.major, self.minor, self.patch,
            releaselevel=self.preRelName, serial=self.preRelease
        )
        return version_dict


class SemVersion(object):
    def __init__(self, major, minor, patch, releaselevel=None, serial=None):
        self.major = major
        self.minor = minor
        self.patch = patch
        self.releaselevel = releaselevel or ""
        self.serial = serial or 0

    def __repr__(self):
        repr_str = "version_info(major={major}, minor={minor}, patch={patch}, " \
               "releaselevel='{releaselevel}', serial={serial})".format(
            major=self.major, minor=self.minor, patch=self.patch,
            releaselevel=self.releaselevel, serial=self.serial
        )
        return repr_str


if __name__ == '__main__':
    version = Version("1.0.1-beta.1")
    print(version.major)
    print(version.preRelease)
    print(version)



from setuptools import setup
from glob import glob
from shutil import rmtree
import os
try:
    from setuptools.command.clean import clean as clean_command
except:
    from distutils.command.clean import clean as clean_command

class Clean(clean_command):
    def run(self):
        clean_command.run(self)
        delete_in_root = ['build', 'dist', '*.egg-info', '.tox', '.eggs', '.cache']
        delete_everywhere = ['__pycache__', '*.pyc']

        for candidate in delete_in_root:
            rmtree_glob(candidate)

        for visible_dir in glob("[A-Za-z0-9]*"):
            for candidate in delete_everywhere:
                rmtree_glob(os.path.join(visible_dir, candidate))
                rmtree_glob(os.path.join(visible_dir, "*", candidate))

def rmtree_glob(candidate):
    for fobj in glob(candidate):
        try:
            rmtree(fobj)
            print("%s/ removed from project" % fobj)
        except OSError:
            try:
                os.remove(fobj)
                print("%s/ removed from project" % fobj)
            except OSError:
                print("Testing")


def read_file(path=None, file_name=None):
    path = path or os.path.dirname(__file__)
    file_path = os.path.join(path, file_name)
    with open(file_path) as fd:
        return fd.read()

packageName = "traVer"
exec(read_file(path="./%s" % packageName, file_name="__init__.py"))
projectName = __name__
version = __version__
readmeData = read_file(file_name="README.md")
licenseData = read_file(file_name="LICENSE")

setup(
    name=packageName,
    description="Semantic versioning for python projects",
    version=version,
    packages=[
        packageName,
    ],
    license="GNU GENERAL PUBLIC LICENSE",
    long_description_content_type='text/markdown',
    long_description=readmeData,
    author=', '.join(__author__),
    author_email=', '.join(__author_email__),
    url="https://github.com/sunilk-n/TraVersion",
    classifiers=[
        # See https://pypi.org/pypi?%3Aaction=list_classifiers
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*",
    data_files=[
        "./README.md",
        "./LICENSE"
    ],
    cmdclass={
        "clean": Clean,
    },
)
